/////////////////////////////////////////////////////////////////////////
// Ping-Hong Lu 
// 
// This program runs on the Nios II processor instantiated on an Altera
// DE2 board.  It receives VGA information from the board and calcuates
// the movement and velocity of a light source. 
/////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include "system.h"
#include "math.h"
#include "altera_avalon_pio_regs.h"

// I like these
#define begin {
#define end }
#define position_delta 0
#define CIR_BUFFER_SIZE 10
#define RADIUS  13
#define LEFT_WALL   (8  + RADIUS)
#define RIGHT_WALL  (620 - RADIUS)
#define TOP_WALL    (100 + RADIUS)
#define BOTTOM_WALL (400 - RADIUS)
#define MAX_SPEED 6
#define MIN_SPEED 0x0080
#define FREEZE_CONST 12


#define INCREASE_1  0x0000
#define INCREASE_2  0x0010
#define INCREASE_3  0x0020
#define INCREASE_4  0x0040
#define INCREASE_5  0x0080
#define INCREASE_6  0x0100
#define INCREASE_7  0x0140
#define INCREASE_8  0x0180
#define INCREASE_9  0x0200
#define INCREASE_10 0x0280
#define INCREASE_11 0x0300
#define INCREASE_12 0x0380
#define INCREASE_13 0x0400
#define INCREASE_14 0x0480
#define INCREASE_15 0x0500
#define INCREASE_16 0x0580
#define INCREASE_17 0x0600
#define INCREASE_18 0x0600


#define float2fix(a) (int)( (a) * 256.0)
#define mfix(a,b) ((int)((((long)(a))*((long)(b)))>>8))


///// LCD control: //////////////////////////////////////////////////////
// From: http://www.altera.com/support/examples/nios2/exm-micro_mutex.html
// ESC ssequences:
// From: http://www.isthe.com/chongo/tech/comp/ansi_escapes.html
// ESC[#;#H      Moves the cursor to line #, column #
// ESC[2J        Clear screen and home cursor
// ESC[K         Clear to end of line
#define ESC_TOP_LEFT    "[1;0H"
#define ESC_BOTTOM_LEFT "[2;0H"
#define LCD_CLR "[2J"
#define LCD_CLR_LINE "[K"
static unsigned char esc = 0x1b; // Integer ASCII value of the ESC character
//File descriptor for the LCD
FILE * lcd_fd;


int main(void)
begin
  int address_x, valid, address_y;
  int address_x_2, address_y_2;
  int x_position1, y_position1;
  int x_position2, y_position2, no_change;
  int sw, key, notecount;
  int left_half = 1;
  int x_diff, y_diff;  // differences in x and y centers of puck & paddle
  int speed_memory_1, speed_memory_2;
  int x_speed_increase, y_speed_increase;
  int friction_distance;
  char collision_flag;
  char speed_x_positive_flag = 1;
  char speed_y_positive_flag = 1;
  unsigned char LEFT_PLAYER_SCORED = 0;
  unsigned char RIGHT_PLAYER_SCORED = 0;
  unsigned char LEFT_SCORE = 0;
  unsigned char RIGHT_SCORE = 0;
  

  int speed_x = 0;
  int speed_y = 0;
  int speed_x_2 = 0;
  int speed_y_2 = 0;
  int speed_x_relevant, speed_y_relevant;
  
  int speed;
  char game_began = 0;
  
  int collision_count = 0;
  unsigned int delay_counter = 0;
  int dummy = 0;
  
  
  unsigned int notes[9]={262, 294, 330, 349, 392, 440, 494, 523, 0};
  unsigned int msg = 0;
  unsigned int freq = 0;
  unsigned int x_position_history[CIR_BUFFER_SIZE];
  unsigned int y_position_history[CIR_BUFFER_SIZE];
  unsigned int head, tail;
  
  unsigned int x_position = 0x1400  + (RADIUS<<8);  // Fit the entire puck within border
  unsigned int y_position = 0x6400 + (RADIUS<<8);  // Fir the entire puck within border
  int x_speed = 0x00000000;
  int y_speed = 0x00000700;
  
  int freeze_collision = 0;
  
  
  // jtag_uart variables
  FILE *uart_file;
  int freq_val;
  int wave_val;
  
  notecount = 0;
  
//  x_position1 = 619;
//  y_position1 = 459;
  no_change = 1;
  
  head = 0;
  tail = 0;
  
  //open the lcd --- device name from system.h
  lcd_fd = fopen("/dev/lcd", "w");
  if(lcd_fd == NULL) printf("Unable to open lcd display\n");


  while (1)
  begin

    if (game_began == 0)
    {
      x_speed = 0x00000000;
      y_speed = 0x00000400;
      x_position = 0x00013100  + (RADIUS<<8);
      y_position = 0x00006400 + (RADIUS<<8);
      IOWR_ALTERA_AVALON_PIO_DATA(OUT9_BASE, 0);
      
      address_x = IORD_ALTERA_AVALON_PIO_DATA(IN0_BASE);    // Paddle 1 position in int
      address_y = IORD_ALTERA_AVALON_PIO_DATA(IN2_BASE);

      address_x_2 = IORD_ALTERA_AVALON_PIO_DATA(IN3_BASE);  // Paddle 2 position in int
      address_y_2 = IORD_ALTERA_AVALON_PIO_DATA(IN4_BASE);

      if (address_x >= 135 && address_x <= 155)
      {
        if (address_y >= 240 && address_y <= 260)
        {
          if (address_x_2 >= 485 && address_x_2 <= 505)
          {
            if (address_y_2 >= 240 && address_y_2 <= 260)
            {
              game_began = 1;
              RIGHT_SCORE = 0;
              LEFT_SCORE = 0;
            }  
          }
        }
      }
    }
  
    if (game_began == 1)
    begin
      IOWR_ALTERA_AVALON_PIO_DATA(OUT9_BASE, 1);

      if (msg == 0) msg = 1;
      else msg = 0;
      if (LEFT_PLAYER_SCORED)
      begin
        IOWR_ALTERA_AVALON_PIO_DATA(OUT10_BASE, 0);
        delay_counter = 0;
        while (delay_counter < 1500000)
        {
          delay_counter++;
          dummy = delay_counter;
        }
        delay_counter = 0;
        x_position = 0x01D600  + (RADIUS<<8);  // Fit the entire puck within border
        y_position = 0x6400 + (RADIUS<<8);  // Fir the entire puck within border
        x_speed = 0x0000;
        y_speed = 0x0300;
        LEFT_PLAYER_SCORED = 0;  
      end
      else if (RIGHT_PLAYER_SCORED)
      begin
        IOWR_ALTERA_AVALON_PIO_DATA(OUT10_BASE, 0);
        delay_counter = 0;
        while (delay_counter < 1500000)
        {
          delay_counter++;
          dummy = delay_counter;
        }
        delay_counter = 0;
        x_position = 0xAA00  + (RADIUS<<8);  // Fit the entire puck within border
        y_position = 0x6400 + (RADIUS<<8);  // Fir the entire puck within border
        x_speed = 0x0000;
        y_speed = 0x0300;
        RIGHT_PLAYER_SCORED = 0;   
      end
    
    
      //msg = (unsigned int)left_half;
    
      IOWR_ALTERA_AVALON_PIO_DATA(OUT0_BASE, msg); 
      IOWR_ALTERA_AVALON_PIO_DATA(OUT7_BASE, 0);
      IOWR_ALTERA_AVALON_PIO_DATA(OUT8_BASE, 0);
      

      // Check the validity of the data
      //valid = IORD_ALTERA_AVALON_PIO_DATA(IN1_BASE); 


      // --------- START OF PING -----------------
      x_position_history[0] = x_position1;  // Previous left paddle x position
      y_position_history[0] = y_position1;  // Previous left paddle y position
 
      x_position_history[1] = x_position2; // Previous right paddle x position
      y_position_history[1] = y_position2; // Previous right paddle y position
    
      address_x = IORD_ALTERA_AVALON_PIO_DATA(IN0_BASE); // Paddle 1 position in int
      address_y = IORD_ALTERA_AVALON_PIO_DATA(IN2_BASE); 

      address_x = address_x << 8; // Paddle 1 position in fixed point
      address_y = address_y << 8; 

      address_x_2 = IORD_ALTERA_AVALON_PIO_DATA(IN3_BASE);  // Paddle 1 position in int
      address_y_2 = IORD_ALTERA_AVALON_PIO_DATA(IN4_BASE);

      address_x_2 = address_x_2 << 8;  // Paddle 2 position in fixed point
      address_y_2 = address_y_2 << 8;

      x_position1 = address_x;
      y_position1 = address_y;
      
      x_position2 = address_x_2;
      y_position2 = address_y_2;
      
      no_change = 0;

      speed = sqrt((double)(
               abs(x_position_history[0] - x_position1) *
               abs(x_position_history[0] - x_position1) +
               abs(y_position_history[0] - y_position1) *
               abs(y_position_history[0] - y_position1)));

      //speed = sqrt((double)(speed));


      if (x_position_history[0] == x_position1 &&
          y_position_history[0] == y_position1)
      begin
        if (speed_memory_1 == 0)
        begin
          speed_x = (x_position_history[0] - x_position1);
          speed_y = (y_position_history[0] - y_position1);
          speed_memory_1 = 100;
        end
        else
          speed_memory_1--;
      end
      else
      begin
        speed_x = (x_position_history[0] - x_position1);
        speed_y = (y_position_history[0] - y_position1);
        speed_memory_1 = 3000;
      end
 
 
      if (x_position_history[1] == x_position2 &&
          y_position_history[1] == y_position2)
      begin
        if (speed_memory_2 == 0)
        begin
          speed_x_2 = (x_position_history[1] - x_position2);
          speed_y_2 = (y_position_history[1] - y_position2);
          speed_memory_2 = 100;
        end
        else
          speed_memory_2--;
      end
      else
      begin
        speed_x_2 = (x_position_history[1] - x_position2);
        speed_y_2 = (y_position_history[1] - y_position2);
        speed_memory_2 = 3000;
      end
 
/*      if (speed_x == 0 && speed_y == 0)
      begin
        if (speed_memory_1 == 0)
        begin
          speed_x = (x_position_history[0] - x_position1);
          speed_y = (y_position_history[0] - y_position1);
          speed_memory_1 = 100;
        end
        else
        begin
          speed_memory_1--;
        end
      end
      else
      begin
        speed_x = (x_position_history[0] - x_position1);
        speed_y = (y_position_history[0] - y_position1);
        speed_memory_1 = 10;
          speed_x = (x_position_history[0] - x_position1);
          speed_y = (y_position_history[0] - y_position1);
          speed_memory_1 = 100;
      end


      if (speed_x_2 == 0 && speed_y_2 == 0)
      begin
        if (speed_memory_2 == 0)
        begin
          speed_x_2 = (x_position_history[1] - x_position2);
//      speed_x_2 = (speed_x_2 >> 2) * 10;
          speed_y_2 = (y_position_history[1] - y_position2);
//      speed_y_2 = (speed_y_2 >> 2) * 10;
          speed_memory_2 = 10;
        end
        else
        begin
          speed_memory_2--;
        end
      end
      else
      begin
        speed_x_2 = (x_position_history[1] - x_position2);
        speed_y_2 = (y_position_history[1] - y_position2);
        speed_memory_2 = 10;
      end
*/
/*      speed_x_2 = sqrt((double)(
               abs(x_position_history[1] - x_position1) *
               abs(x_position_history[1] - x_position1) +
               abs(y_position_history[1] - y_position1) *
               abs(y_position_history[1] - y_position1)));
      speed2 = speed2 / 3.1 * 1000000;
*/
      // commented out because printf takes a loooong time
      // will use LEDR, and HEX0-7 instead
//        printf("x: %d\ny: %d\nspeed: %d\n",
//                address_x, address_y, speed);

        IOWR_ALTERA_AVALON_PIO_DATA(OUT2_BASE, LEFT_SCORE);
        IOWR_ALTERA_AVALON_PIO_DATA(OUT3_BASE, RIGHT_SCORE);
//        IOWR_ALTERA_AVALON_PIO_DATA(OUT3_BASE, address_x);
//        IOWR_ALTERA_AVALON_PIO_DATA(OUT4_BASE, address_y);
        
        speed_x_relevant = ((left_half == 1) ? speed_x :
                                               speed_x_2);
        speed_y_relevant = ((left_half == 1) ? speed_y :
                                               speed_y_2);
                                                       
        if (abs(speed_x_relevant) < 1)        // 1 LED
        begin
          IOWR_ALTERA_AVALON_PIO_DATA(OUT1_BASE, 1);
          x_speed_increase = INCREASE_1;
        end
        else if (abs(speed_x_relevant) < 1000)  // 2 LED
        begin
          IOWR_ALTERA_AVALON_PIO_DATA(OUT1_BASE, 3);
          x_speed_increase = INCREASE_2;
        end
        else if (abs(speed_x_relevant) < 2000)  // 3 LED
        begin
          IOWR_ALTERA_AVALON_PIO_DATA(OUT1_BASE, 7);
          x_speed_increase = INCREASE_3;
        end
        else if (abs(speed_x_relevant) < 3000)  // 4 LED
        begin
          IOWR_ALTERA_AVALON_PIO_DATA(OUT1_BASE, 15);
          x_speed_increase = INCREASE_4;
        end
        else if (abs(speed_x_relevant) < 4000)  // 5 LED
        begin
          IOWR_ALTERA_AVALON_PIO_DATA(OUT1_BASE, 31);
          x_speed_increase = INCREASE_5;
        end
        else if (abs(speed_x_relevant) < 5000)  // 6 LED
        begin
          IOWR_ALTERA_AVALON_PIO_DATA(OUT1_BASE, 63);
          x_speed_increase = INCREASE_6;
        end
        else if (abs(speed_x_relevant) < 6000)  // 7 LED
        begin
          IOWR_ALTERA_AVALON_PIO_DATA(OUT1_BASE, 127);
          x_speed_increase = INCREASE_7;
        end
        else if (abs(speed_x_relevant) < 7000)  // 8 LED
        begin
          IOWR_ALTERA_AVALON_PIO_DATA(OUT1_BASE, 255);
          x_speed_increase = INCREASE_8;
        end
        else if (abs(speed_x_relevant) < 8000)  // 9 LED
        begin
          IOWR_ALTERA_AVALON_PIO_DATA(OUT1_BASE, 511);
          x_speed_increase = INCREASE_9;
        end
        else if (abs(speed_x_relevant) < 9000)  // 10 LED
        begin
          IOWR_ALTERA_AVALON_PIO_DATA(OUT1_BASE, 1023);
          x_speed_increase = INCREASE_10;
        end
        else if (abs(speed_x_relevant) < 10000)  // 11 LED
        begin
          IOWR_ALTERA_AVALON_PIO_DATA(OUT1_BASE, 2047);
          x_speed_increase = INCREASE_11;
        end
        else if (abs(speed_x_relevant) < 11000)  // 12 LED
        begin
          IOWR_ALTERA_AVALON_PIO_DATA(OUT1_BASE, 4095);
          x_speed_increase = INCREASE_12;
        end
        else if (abs(speed_x_relevant) < 12000)  // 13 LED
        begin
          IOWR_ALTERA_AVALON_PIO_DATA(OUT1_BASE, 8191);
          x_speed_increase = INCREASE_13;
        end
        else if (abs(speed_x_relevant) < 13000)  // 14 LED
        begin
          IOWR_ALTERA_AVALON_PIO_DATA(OUT1_BASE, 16383);
          x_speed_increase = INCREASE_14;
        end
        else if (abs(speed_x_relevant) < 14000)  // 15 LED
        begin
          IOWR_ALTERA_AVALON_PIO_DATA(OUT1_BASE, 32767);
          x_speed_increase = INCREASE_15;
        end
        else if (abs(speed_x_relevant) < 15000)  // 16 LED
        begin
          IOWR_ALTERA_AVALON_PIO_DATA(OUT1_BASE, 65535);
          x_speed_increase = INCREASE_16;
        end
        else if (abs(speed_x_relevant) < 30000)  // 17 LED
        begin
          IOWR_ALTERA_AVALON_PIO_DATA(OUT1_BASE, 131071);
          x_speed_increase = INCREASE_17;
        end
        else                   // 18 LED
        begin
          IOWR_ALTERA_AVALON_PIO_DATA(OUT1_BASE, 262143);
          x_speed_increase = INCREASE_18;
        end

        if (abs(speed_y_relevant) < 1)        // 1 LED
        begin
          y_speed_increase = INCREASE_1;
        end
        else if (abs(speed_y_relevant) < 1000)  // 2 LED
        begin
          y_speed_increase = INCREASE_2;
        end
        else if (abs(speed_y_relevant) < 2000)  // 3 LED
        begin
          y_speed_increase = INCREASE_3;
        end
        else if (abs(speed_y_relevant) < 3000)  // 4 LED
        begin
          y_speed_increase = INCREASE_4;
        end
        else if (abs(speed_y_relevant) < 4000)  // 5 LED
        begin
          y_speed_increase = INCREASE_5;
        end
        else if (abs(speed_y_relevant) < 5000)  // 6 LED
        begin
          y_speed_increase = INCREASE_6;
        end
        else if (abs(speed_y_relevant) < 6000)  // 7 LED
        begin
          y_speed_increase = INCREASE_7;
        end
        else if (abs(speed_y_relevant) < 7000)  // 8 LED
        begin
          y_speed_increase = INCREASE_8;
        end
        else if (abs(speed_y_relevant) < 8000)  // 9 LED
        begin
          y_speed_increase = INCREASE_9;
        end
        else if (abs(speed_y_relevant) < 9000)  // 10 LED
        begin
          y_speed_increase = INCREASE_10;
        end
        else if (abs(speed_y_relevant) < 10000)  // 11 LED
        begin
          y_speed_increase = INCREASE_11;
        end
        else if (abs(speed_y_relevant) < 1100)  // 12 LED
        begin
          y_speed_increase = INCREASE_12;
        end
        else if (abs(speed_y_relevant) < 12000)  // 13 LED
        begin
          y_speed_increase = INCREASE_13;
        end
        else if (abs(speed_y_relevant) < 13000)  // 14 LED
        begin
          y_speed_increase = INCREASE_14;
        end
        else if (abs(speed_y_relevant) < 14000)  // 15 LED
        begin
          y_speed_increase = INCREASE_15;
        end
        else if (abs(speed_y_relevant) < 15000)  // 16 LED
        begin
          y_speed_increase = INCREASE_16;
        end
        else if (abs(speed_y_relevant) < 30000)  // 17 LED
        begin
          y_speed_increase = INCREASE_17;
        end
        else                   // 18 LED
        begin
          y_speed_increase = INCREASE_18;
        end

    if (((x_diff>>8) * (x_diff>>8) + (y_diff>>8) * (y_diff>>8)) <=
        (4 * (RADIUS - 1) * (RADIUS - 1)))  // Radius - 1 makes collision closer
      if (!freeze_collision)
        collision_flag = 1;



    // DYNAMICS
    if (notecount++ == 200 )   // slowing down puck movement
    begin
      if (freeze_collision > 0) freeze_collision--;
      
      x_position_history[2] = x_position;      // Previous puck x position
      y_position_history[2] = y_position;      // Previous puck y position

      if (speed_x_positive_flag == 1)
        x_position += x_speed;  // x coordinate of puck
      else
        x_position -= x_speed;  // x coordinate of puck
      if (speed_y_positive_flag == 1)
        y_position += y_speed;  // y coordinate of puck
      else
        y_position -= y_speed;  // y coordinate of puck
      
      x_diff = (left_half == 1) ? (x_position - x_position1) : 
                                  (x_position - x_position2);
      y_diff = (left_half == 1) ? (y_position - y_position1) : 
                                  (y_position - y_position2);
      // RIGHT WALL 
      if ((y_position < ((225<<8)+(RADIUS<<8)) || y_position >= ((285<<8)-(RADIUS<<8))) && 
          (x_position >= (RIGHT_WALL<<8)))
      begin
        if (speed_x_positive_flag == 1)
        begin
          if (x_speed > MIN_SPEED)
            x_speed -= 0x0040;
          speed_x_positive_flag = 0;
        end  
        IOWR_ALTERA_AVALON_PIO_DATA(OUT7_BASE, 1);
      end
      // RIGHT GOAL
      if ((y_position >= ((225<<8)+(RADIUS<<8)) && y_position < ((285<<8)-(RADIUS<<8))) && 
          (x_position >= ((RIGHT_WALL+RADIUS+1)<<8)))
      begin
        
                        /*if (speed_x_positive_flag == 1)
        begin
          if (x_speed > 0x0040)
            x_speed -= 0x0040;
          speed_x_positive_flag = 0;
        end  
        IOWR_ALTERA_AVALON_PIO_DATA(OUT7_BASE, 1);
        */
        // Generate chime
        IOWR_ALTERA_AVALON_PIO_DATA(OUT10_BASE, 1);
        LEFT_PLAYER_SCORED = 1;
        LEFT_SCORE++;
        if (LEFT_SCORE == 10)
        {
          game_began = 0;    
        }
      end
      // LEFT WALL
      if ((y_position < ((225<<8)+(RADIUS<<8)) || y_position >= ((285<<8)-(RADIUS<<8))) && 
          (x_position <= ((LEFT_WALL+RADIUS+1)<<8)))
      begin
        if (speed_x_positive_flag == 0)
        begin
          if (x_speed > MIN_SPEED)
            x_speed -= 0x0040;
          speed_x_positive_flag = 1;
        end
        IOWR_ALTERA_AVALON_PIO_DATA(OUT7_BASE, 1);
      end
      // LEFT GOAL
      if ((y_position >= ((225<<8)+(RADIUS<<8)) && y_position < ((285<<8)-(RADIUS<<8))) && 
          (x_position <= (LEFT_WALL<<8)))
      begin

                         /*if (speed_x_positive_flag == 0)
        begin
          if (x_speed > 0x0040)
            x_speed -= 0x0040;
          speed_x_positive_flag = 1;
        end
        IOWR_ALTERA_AVALON_PIO_DATA(OUT7_BASE, 1);        
        */
        // Generate chime
        IOWR_ALTERA_AVALON_PIO_DATA(OUT10_BASE, 1);
        RIGHT_PLAYER_SCORED = 1;
        RIGHT_SCORE++;
        if (RIGHT_SCORE == 10)
        {
          game_began = 0;    
        }
      end
      // BOTTOM WALL
      if (y_position >= (BOTTOM_WALL<<8))
      begin
        if (speed_y_positive_flag == 1)
        begin
          if (y_speed > MIN_SPEED)
            y_speed -= 0x0040;
          speed_y_positive_flag = 0;
        end
        IOWR_ALTERA_AVALON_PIO_DATA(OUT7_BASE, 1);
      end
      // TOP WALL
      if (y_position <= (TOP_WALL<<8))
      begin
        if (speed_y_positive_flag == 0)
        begin
          if (y_speed > MIN_SPEED)
            y_speed -= 0x0040;
          speed_y_positive_flag = 1;
        end
        IOWR_ALTERA_AVALON_PIO_DATA(OUT7_BASE, 1);
      end

      if (collision_flag == 1)
      // PADDLES
      begin                                   // to look convincing on video
        collision_count++;
        //printf("collision occurred with speed increase of %d %d\n speed x = %d speed y = %d collision count = %d\n", 
        //    x_speed_increase, y_speed_increase, speed_x_relevant, speed_y_relevant, collision_count);
      
        collision_flag = 0;
        // Collision sound effect
        IOWR_ALTERA_AVALON_PIO_DATA(OUT8_BASE, 1);
        // COLLISION 1 
        if (speed_x_positive_flag == 1 && 
            speed_y_positive_flag == 1 && !freeze_collision)
        begin
          // bounce up & left if puck is above & left of paddle 
          if (y_diff < 0 && x_diff < 0)
          begin
            speed_x_positive_flag = 0;
            speed_y_positive_flag = 0;
          end
          // bounce up & right if puck is above & right of paddle
          else if(y_diff < 0 && x_diff >= 0)
          begin
            speed_y_positive_flag = 0;
          end
          // bounce down & left if puck is below paddle
          else if(y_diff >= 0 && x_diff < 0)
          begin
            speed_x_positive_flag = 0;
          end
          else
          begin
            // same direction
          end
          
          freeze_collision = FREEZE_CONST;
        end
        // COLLISION 2
        if (speed_x_positive_flag == 1 && 
            speed_y_positive_flag == 0 && !freeze_collision)
        begin
          // bounce down & left if puck is below & left of paddle
          if (y_diff > 0 && x_diff < 0)
          begin
            speed_x_positive_flag = 0;
            speed_y_positive_flag = 1;
          end
          // bounce down & right if puck is above & left of paddle 
          else if (y_diff > 0 && x_diff >= 0)
          begin
            speed_y_positive_flag = 1;
          end
          // bounce up & left if puck is above & left of paddle
          else if(y_diff <= 0 && x_diff < 0)
          begin
            speed_x_positive_flag = 0;
          end
          else
          begin
            // same direction
          end
          freeze_collision = FREEZE_CONST;
        end
        // COLLISION 3
        if (speed_x_positive_flag == 0 && 
            speed_y_positive_flag == 1 && !freeze_collision)
        begin

          // bounce up & right if puck is above & right of paddle
          if (y_diff < 0 && x_diff > 0)
          begin
            speed_x_positive_flag = 1;
            speed_y_positive_flag = 0;
          end
          // bounce up & left if puck is above & left of paddle 
          else if (y_diff < 0 && x_diff <= 0)
          begin
            speed_y_positive_flag = 0;
          end
          // bounce down & right if puck is below & right of paddle
          else if(y_diff >= 0 && x_diff > 0)
          begin
            speed_x_positive_flag = 1;
          end
          else
          begin
            // same direction
          end
          freeze_collision = FREEZE_CONST;
        end
        // COLLISION 4
        if (speed_x_positive_flag == 0 && 
            speed_y_positive_flag == 0 && !freeze_collision)
        begin
          // bounce down & right if puck is below & right of paddle
          if (y_diff > 0 && x_diff > 0)
          begin
            speed_x_positive_flag = 1;
            speed_y_positive_flag = 1;
          end
          // bounce down & left if puck is below & left of paddle 
          else if (y_diff > 0 && x_diff <= 0)
          begin
            speed_y_positive_flag = 1;
          end
          // bounce up & right if puck is above & right of paddle
          else if (y_diff <= 0 && x_diff > 0)
          begin
            speed_x_positive_flag = 1;
          end
          else
          begin
            // same direction 
          end
          freeze_collision = FREEZE_CONST;
        end
        
        if (x_speed == 0)
        {
             if (abs(x_diff) <= 1)
            {
                // do nothing
            }
            else if (abs(x_diff) <= 4)
            {
                x_speed = MIN_SPEED;
                y_speed -= 0x0040;
            }
            else if (abs(x_diff) <= 8)
            {
                x_speed = 0x0050;
                y_speed -= 0x0050;
            }
            else
            {
                x_speed = 0x0060;
                y_speed -= 0x0060;
            }
        }
        
        // ALL COLLISIONS DONE, CHANGE SPEEDS
        if ((x_speed + x_speed_increase) <= (MAX_SPEED<<8)) 
        begin
          if (x_speed > 0x0500)
          begin
            x_speed += (x_speed_increase / 2);
          end
          else
            x_speed += x_speed_increase;
        end
        else
          x_speed = MAX_SPEED<<8;

        if ((y_speed - y_speed_increase) < (MAX_SPEED<<8))
        begin
          if (y_speed > 0x0500)
          begin
            y_speed += (y_speed_increase / 2);
          end
          else
            y_speed += y_speed_increase;
        end
        else
          y_speed = MAX_SPEED<<8;


/*        // FRICTION
        if (friction_distance++ > 19)
        begin
          friction_distance = 0;
          if (x_speed > 0x0040)
            x_speed-=0x0002;
          if (y_speed > 0x0040) 
            y_speed-=0x0002;
        end
*/        
      end

      notecount = 0;
    end
 
    if ((x_position>>8) < 320)
    begin
      // the puck is in the left half
      left_half = 1;
    end
    else if ((x_position>>8) >= 320)
    begin
      // the puck is in the right half
      left_half = 0;
    end



    IOWR_ALTERA_AVALON_PIO_DATA(OUT4_BASE, 0);
    
    IOWR_ALTERA_AVALON_PIO_DATA(OUT5_BASE, x_position>>8);
    IOWR_ALTERA_AVALON_PIO_DATA(OUT6_BASE, y_position>>8);
    


    end
  end
end